use sugar bam;
my $x = 11;
print refaddr( \$x );
print "MAX=" . max( 1..20 );
cluck "okkk";
#my $d = { aa=>{ bb=> { cc => { name=>'rod' } } } };
#alias my $obj = $d->{aa}->{bb}->{cc};
#print $obj->{name};
